from pika_client.connector import _AsyncConnector


class AsyncConsumer(_AsyncConnector):
    def __init__(self, connection_parameters, **kwargs):
        self._queue = kwargs.get("queue")
        self._on_message = self._on_message_ack_decorator(on_message)
        super(AsyncConsumer, self).__init__(connection_parameters, self._start_consume, **kwargs)
        self._consumer_tag = None

    def start_operation(self):
        self._start_consume(self._channel)

    def _on_message_ack_decorator(self, on_message):
        def __on_message_ack_decorator(channel, basic_deliver, properties, body):
            try:
                on_message(body)
                channel.basic_ack(basic_deliver.delivery_tag)
            except:
                channel.basic_nack(basic_deliver.delivery_tag)
        return __on_message_ack_decorator

    def _start_consume(self, channel):
        self._channel.add_on_cancel_callback(self.on_consumer_cancelled)
        self._consumer_tag = self._channel.basic_consume(self._on_message, self._queue)

    def on_consumer_cancelled(self, method_frame):
        if self._channel:
            self._channel.close()

    def stop(self):
        if self._stopping:
            return
        self._stopping = True
        self._stop_consuming()
        self._connection.ioloop.start()

    def _stop_consuming(self):
        if self._channel:
            self._channel.basic_cancel(self._on_cancelok, self._consumer_tag)

    def _on_cancelok(self, unused_frame):
        self._channel.close()



from pika_client.route_setup import RouteSetup


class Launcher(object):
    def __init__(self, connector, route_config=None):
        self._connector = connector
        self._route_config = route_config

    def _launch(self):
        self._connector.add_listener(self)
        self._connector.start()

    def _on_channel_open(self, channel):
        if self._route_config is not None:
            route_setup = RouteSetup(self._route_config, channel, self._on_route_setup_complete)
            route_setup.setup()
        else:
            self._on_route_setup_complete()

    def _on_route_setup_complete(self):
        self._connector.start_operation()

    def start(self):
        self._launch()

    def __getattr__(self, name):
        return getattr(self._connector, name)



import pika
import threading


class _AsyncConnector(threading.Thread):
    def __init__(self, connection_parameters, **kwargs):
        super(_AsyncConnector, self).__init__()
        user = connection_parameters.pop("user", "guest")
        password = connection_parameters.pop("password", "guest")
        connection_parameters["credentials"] = pika.PlainCredentials(username=user, password=password)
        self._connection_parameters = connection_parameters
        self._connection = None
        self._channel = None
        self._stopping = False
        self._listeners = []

    def add_listener(self, l):
        self._listeners.append(l)

    def remove_listener(self, l):
        self._listeners.remove(l)

    def _on_connection_open(self, connection):
        self._connection.channel(on_open_callback=self._on_channel_open)

    def _event_on_channel_open(self, channel):
        for l in self._listeners:
            l._on_channel_open(channel)

    def _on_channel_open(self, channel):
        self._channel = channel
        self._channel.add_on_close_callback(self._on_channel_closed)
        self._event_on_channel_open(channel)

    def _on_channel_closed(self, channel, reply_code, reply_text):
        self._close_connection()

    def start_operation(self):
        raise NotImplementedError("must implement in derived class")

    def run(self):
        self._connection = pika.SelectConnection(
            pika.ConnectionParameters(**self._connection_parameters),
            self._on_connection_open, self.on_open_error, self.on_connection_closed,
            stop_ioloop_on_close=False)
        self._connection.ioloop.start()

    def on_connection_closed(self, connection, reply_code, reply_text):
        self._connection.ioloop.stop()

    def on_open_error(self, connection, reply_code, reply_text):
        self._connection.ioloop.stop()

    def _close_connection(self):
        if self._connection is not None:
            self._connection.close()

    def stop(self):
        if self._stopping:
            return
        self._stopping = True
        self._close_connection()
